from __future__ import annotations

import contextlib
import hashlib
import os
import re
import warnings
from textwrap import dedent
from typing import Callable, Iterable

import branca
import folium
import folium.elements
import folium.plugins
import streamlit as st
import streamlit.components.v1 as components
from jinja2 import UndefinedError

# Create a _RELEASE constant. We'll set this to False while we're developing
# the component, and True when we're ready to package and distribute it.
_RELEASE = True


if not _RELEASE:
    _component_func = components.declare_component(
        "st_folium", url="http://localhost:3001"
    )

else:
    parent_dir = os.path.dirname(os.path.abspath(__file__))
    build_dir = os.path.join(parent_dir, "frontend/build")
    _component_func = components.declare_component("st_folium", path=build_dir)


def generate_js_hash(
    js_string: str, key: str | None = None, return_on_hover: bool = False
) -> str:
    """
    Generate a standard key from a javascript string representing a series
    of folium-generated leaflet objects by replacing the hash's at the end
    of variable names (e.g. "marker_5f9d46..." -> "marker"), and returning
    the hash.

    Also strip maps/<random_hash>, which is generated by google earth engine
    """
    pattern = r"(_[a-z0-9]+)"
    standardized_js = re.sub(pattern, "", js_string) + str(key)
    url_pattern = r"(maps\/[-a-z0-9]+\/)"
    standardized_js = (
        re.sub(url_pattern, "", standardized_js) + str(key) + str(return_on_hover)
    )
    return hashlib.sha256(standardized_js.encode()).hexdigest()


def folium_static(
    fig: folium.Figure | folium.Map,
    width: int | None = 700,
    height: int = 500,
):
    """
    Renders `folium.Figure` or `folium.Map` in a Streamlit app. This method is
    a static Streamlit Component, meaning, no information is passed back from
    Leaflet on browser interaction.
    Parameters
    ----------
    fig  : folium.Map or folium.Figure
        Geospatial visualization to render
    width : int
        Width of result
    Height : int
        Height of result
    Note
    ----
    If `height` is set on a `folium.Map` or `folium.Figure` object,
    that value supersedes the values set with the keyword arguments of this function.

    Example
    -------
    >>> m = folium.Map(location=[45.5236, -122.6750])
    >>> folium_static(m)
    """
    warnings.warn(
        dedent(
            """
        folium_static is deprecated and will be removed in a future release, or
        simply replaced with with st_folium which always passes
        returned_objects=[] to the component.
        Please try using st_folium instead, and
        post an issue at https://github.com/randyzwitch/streamlit-folium/issues
        if you experience issues with st_folium.
        """
        ),
        DeprecationWarning,
        stacklevel=2,
    )
    # if Map, wrap in Figure
    if isinstance(fig, folium.Map):
        fig = folium.Figure().add_child(fig)
        return components.html(
            fig.render(), height=(fig.height or height) + 10, width=width
        )

    # if DualMap, get HTML representation
    if isinstance(fig, (folium.plugins.DualMap, branca.element.Figure)):
        return components.html(fig._repr_html_(), height=height + 10, width=width)
    return st_folium(fig, width=width, height=height, returned_objects=[])


def _get_siblings(fig: folium.MacroElement) -> str:
    """Get the html for any siblings of the map"""
    children = list(fig.get_root()._children.values())

    html = ""
    if len(children) > 1:
        for child in children[1:]:
            with contextlib.suppress(Exception):
                html += child._template.module.html() + "\n"

    return html


def get_full_id(m: folium.MacroElement) -> str:
    if isinstance(m, folium.plugins.DualMap):
        m = m.m1

    return f"{m._name.lower()}_{m._id}"


def _get_map_string(fig: folium.Map) -> str:
    leaflet = generate_leaflet_string(fig)

    # Get rid of the annoying popup
    leaflet = leaflet.replace("alert(coords);", "")

    # Rename drawnItems
    leaflet = re.sub(r"drawnItems_draw_control_div_\d+", "drawnItems", leaflet)

    leaflet = dedent(leaflet)

    if "drawnItems" not in leaflet:
        leaflet += "\nvar drawnItems = [];"

    # Replace the folium generated map_{random characters} variables
    # with map_div and map_div2 (these end up being both the assumed)
    # div id where the maps are inserted into the DOM, and the names of
    # the variables themselves.
    if isinstance(fig, folium.plugins.DualMap):
        m2_id = get_full_id(fig.m2)
        leaflet = leaflet.replace(m2_id, "map_div2")

    return leaflet


def _get_feature_group_string(
    feature_group_to_add: folium.FeatureGroup,
    map: folium.Map,
    idx: int = 0,
) -> str:
    feature_group_to_add._id = f"feature_group_{idx}"
    feature_group_to_add.add_to(map)
    feature_group_to_add.render()
    feature_group_string = generate_leaflet_string(
        feature_group_to_add, base_id=f"feature_group_{idx}"
    )
    m_id = get_full_id(map)
    feature_group_string = feature_group_string.replace(m_id, "map_div")
    feature_group_string = dedent(feature_group_string)

    feature_group_string += dedent(
        f"""
        map_div.addLayer(feature_group_feature_group_{idx});
        window.feature_group = window.feature_group || [];
        window.feature_group.push(feature_group_feature_group_{idx});
        """
    )

    return feature_group_string


def _get_layer_control_string(
    control: folium.LayerControl,
    map: folium.Map,
) -> str:
    control._id = "layer_control"
    control.add_to(map)
    control.render()
    control_string = generate_leaflet_string(control, base_id="layer_control")
    m_id = get_full_id(map)
    control_string = control_string.replace(m_id, "map_div")
    control_string = dedent(control_string)
    control_string += dedent(
        """
        window.layer_control = layer_control_layer_control;
        """
    )

    return control_string


def st_folium(
    fig: folium.MacroElement,
    key: str | None = None,
    height: int = 700,
    width: int | None = 500,
    returned_objects: Iterable[str] | None = None,
    zoom: int | None = None,
    center: tuple[float, float] | None = None,
    feature_group_to_add: list[folium.FeatureGroup] | folium.FeatureGroup | None = None,
    return_on_hover: bool = False,
    use_container_width: bool = False,
    layer_control: folium.LayerControl | None = None,
    pixelated: bool = False,
    debug: bool = False,
    render: bool = True,
    on_change: Callable | None = None,
):
    """Display a Folium object in Streamlit, returning data as user interacts
    with app.
    Parameters
    ----------
    fig  : folium.Map or folium.Figure
        Geospatial visualization to render
    key: str or None
        An optional key that uniquely identifies this component. If this is
        None, and the component's arguments are changed, the component will
        be re-mounted in the Streamlit frontend and lose its current state.
    returned_objects: Iterable
        A list of folium objects (as keys of the returned dictionary) that will be
        returned to the user when they interact with the map. If None, all folium
        objects will be returned. This is mainly useful for when you only want your
        streamlit app to rerun under certain conditions, and not every time the user
        interacts with the map. If an object not in returned_objects changes on the map,
        the app will not rerun.
    zoom: int or None
        The zoom level of the map. If None, the zoom level will be set to the
        default zoom level of the map. NOTE that if this zoom level is changed, it
        will *not* reload the map, but simply dynamically change the zoom level.
    center: tuple(float, float) or None
        The center of the map. If None, the center will be set to the default
        center of the map. NOTE that if this center is changed, it will *not* reload
        the map, but simply dynamically change the center.
    feature_group_to_add: List[folium.FeatureGroup] or folium.FeatureGroup or None
        If you want to dynamically add features to a feature group, you can pass
        the feature group here. NOTE that if you add a feature to the map, it
        will *not* reload the map, but simply dynamically add the feature.
    return_on_hover: bool
        If True, the app will rerun when the user hovers over the map, not
        just when they click on it. This is useful if you want to dynamically
        update your app based on where the user is hovering. NOTE: This may cause
        performance issues if the app is rerunning too often.
    use_container_width: bool
        If True, set the width of the map to the width of the current container.
        This overrides the `width` parameter.
    layer_control: folium.LayerControl or None
        If you want to have layer control for dynamically added layers, you can
        pass the layer control here.
    pixelated: bool
        If True, add CSS rules to render image crisp pixels which gives a pixelated
        result instead of a blurred image.
    debug: bool
        If True, print out the html and javascript code used to render the map with
        st.code
    render: bool
        If True, the map will be rendered as html, this must be done at least once.
        Disabling this may improve performance as you can cache the rendering step.
        *Note* if this is disabled and the map is not rendered elsewhere the map
        will be missing attributes
    Returns
    -------
    dict
        Selected data from Folium/leaflet.js interactions in browser
    """
    # Call through to our private component function. Arguments we pass here
    # will be sent to the frontend, where they'll be available in an "args"
    # dictionary.
    #
    # "default" is a special argument that specifies the initial return
    # value of the component before the user has interacted with it.

    if use_container_width:
        width = None

    folium_map: folium.Map = fig  # type: ignore
    if render:
        folium_map.render()

    # handle the case where you pass in a figure rather than a map
    # this assumes that a map is the first child
    if not (isinstance(fig, (folium.Map, folium.plugins.DualMap))):
        folium_map = next(iter(fig._children.values()))

    folium_map.render()

    leaflet = _get_map_string(folium_map)  # type: ignore

    html = _get_siblings(folium_map)

    m_id = get_full_id(folium_map)

    def bounds_to_dict(bounds_list: list[list[float]]) -> dict[str, dict[str, float]]:
        southwest, northeast = bounds_list
        return {
            "_southWest": {
                "lat": southwest[0],
                "lng": southwest[1],
            },
            "_northEast": {
                "lat": northeast[0],
                "lng": northeast[1],
            },
        }

    try:
        bounds = folium_map.get_bounds()
    except AttributeError:
        bounds = [[None, None], [None, None]]

    _defaults = {
        "last_clicked": None,
        "last_object_clicked": None,
        "last_object_clicked_tooltip": None,
        "last_object_clicked_popup": None,
        "all_drawings": None,
        "last_active_drawing": None,
        "bounds": bounds_to_dict(bounds),
        "zoom": folium_map.options.get("zoom")
        if hasattr(folium_map, "options")
        else {},
        "last_circle_radius": None,
        "last_circle_polygon": None,
        "selected_layers": None,
    }

    # If the user passes a custom list of returned objects, we'll only return those

    defaults = {
        k: v
        for k, v in _defaults.items()
        if returned_objects is None or k in returned_objects
    }

    # Convert the feature group to a javascript string which can be used to create it
    # on the frontend.
    feature_group_string = None
    if feature_group_to_add is not None:
        if isinstance(feature_group_to_add, folium.FeatureGroup):
            feature_group_to_add = [feature_group_to_add]
        feature_group_string = ""
        for idx, feature_group in enumerate(feature_group_to_add):
            feature_group_string += _get_feature_group_string(
                feature_group,
                map=folium_map,
                idx=idx,
            )

    layer_control_string = None
    if layer_control is not None:
        layer_control_string = _get_layer_control_string(layer_control, folium_map)

    if debug:
        with st.expander("Show generated code"):
            if html:
                st.info("HTML:")
                st.code(html)

            st.info("Main Map Leaflet js:")
            st.code(leaflet)

            if feature_group_string is not None:
                st.info("Feature group js:")
                st.code(feature_group_string)

            if layer_control_string is not None:
                st.info("Layer control js:")
                st.code(layer_control_string)

    def walk(fig):
        if isinstance(fig, branca.colormap.ColorMap):
            yield fig
        if isinstance(fig, folium.plugins.DualMap):
            yield from walk(fig.m1)
            yield from walk(fig.m2)
        if isinstance(fig, folium.elements.JSCSSMixin):
            yield fig
        if hasattr(fig, "_children"):
            for child in fig._children.values():
                yield from walk(child)

    css_links: list[str] = []
    js_links: list[str] = []

    for elem in walk(folium_map):
        if isinstance(elem, branca.colormap.ColorMap):
            # manually add d3.js
            js_links.insert(
                0, "https://cdnjs.cloudflare.com/ajax/libs/d3/3.5.5/d3.min.js"
            )
            js_links.insert(0, "https://d3js.org/d3.v4.min.js")
        css_links.extend([href for _, href in getattr(elem, "default_css", [])])
        js_links.extend([src for _, src in getattr(elem, "default_js", [])])

    hash_key = generate_js_hash(leaflet, key, return_on_hover)

    def _on_change():
        if key is not None:
            st.session_state[key] = st.session_state.get(hash_key, {})
        if on_change is not None:
            on_change()

    return _component_func(
        script=leaflet,
        html=html,
        id=m_id,
        key=hash_key,
        height=height,
        width=width,
        returned_objects=returned_objects,
        default=defaults,
        zoom=zoom,
        center=center,
        feature_group=feature_group_string,
        return_on_hover=return_on_hover,
        layer_control=layer_control_string,
        pixelated=pixelated,
        css_links=css_links,
        js_links=js_links,
        on_change=_on_change,
    )


def _generate_leaflet_string(
    m: folium.MacroElement,
    nested: bool = True,
    base_id: str = "0",
    mappings: dict[str, str] | None = None,
) -> tuple[str, dict[str, str]]:
    if mappings is None:
        mappings = {}

    mappings[m._id] = base_id
    try:
        element_id = m.element_name.replace("map_", "").replace("tile_layer_", "")
        parent_id = m.element_parent_name.replace("map_", "").replace("tile_layer_", "")
        if element_id not in mappings:
            mappings[element_id] = m._parent._id
        if parent_id not in mappings:
            mappings[parent_id] = m._parent._parent._id
    except AttributeError:
        pass

    m._id = base_id

    if isinstance(m, folium.plugins.DualMap):
        m.render()
        m.m1.render()
        m.m2.render()
        if not nested:
            return _generate_leaflet_string(
                m.m1, nested=False, mappings=mappings, base_id=base_id
            )
        # Generate the script for map1
        leaflet, _ = _generate_leaflet_string(
            m.m1, nested=nested, mappings=mappings, base_id=base_id
        )
        # Add the script for map2
        leaflet += (
            "\n"
            + _generate_leaflet_string(
                m.m2, nested=nested, mappings=mappings, base_id="div2"
            )[0]
        )
        # Add the script that syncs them together
        leaflet += m._template.module.script(m)
        return leaflet, mappings

    try:
        leaflet = m._template.module.script(m)
    except UndefinedError:
        # Correctly render Popup elements, and perhaps others. Not sure why
        # this is necessary. Some deep magic related to jinja2 templating, perhaps.
        leaflet = m._template.render(this=m, kwargs={})

    if not nested:
        return leaflet, mappings

    for idx, child in enumerate(m._children.values()):
        with contextlib.suppress(UndefinedError, AttributeError):
            leaflet += (
                "\n"
                + _generate_leaflet_string(
                    child, base_id=f"{base_id}_{idx}", mappings=mappings
                )[0]
            )

    return leaflet, mappings


_FOLIUM_VAR_SUFFIX_PATTERN = re.compile("_[a-z0-9]+(?!_)")


def _replace_folium_vars(leaflet: str, mappings: dict[str, str]) -> str:
    def replace(match: re.Match):
        match_str = match.group()
        leaflet_id = match_str.strip("_")
        replacement = mappings.get(leaflet_id)
        if replacement:
            match_str = match_str.replace(leaflet_id, replacement)
        return match_str

    return _FOLIUM_VAR_SUFFIX_PATTERN.sub(replace, leaflet)


def generate_leaflet_string(
    m: folium.MacroElement, nested: bool = True, base_id: str = "div"
) -> str:
    """
    Call the _generate_leaflet_string function, and then replace the
    folium generated var {thing}_{random characters} variables with
    standardized variables, in case any didn't already get replaced
    (e.g. in the case of a LayerControl, it still has a reference
    to the old variable for the tile_layer_{random_characters}).

    This also allows the output to be more testable, since the
    variable names are consistent.
    """
    leaflet, mappings = _generate_leaflet_string(m, nested=nested, base_id=base_id)

    return _replace_folium_vars(leaflet, mappings)



from textwrap import dedent


def test_map():
    import folium

    from streamlit_folium import _get_map_string

    map = folium.Map()
    map.render()

    leaflet = _get_map_string(map)
    assert (
        dedent(
            """var map_div = L.map(
                "map_div",
                {
                    center: [0.0, 0.0],
                    crs: L.CRS.EPSG3857,"""
        )
        in leaflet
    )

    assert "var tile_layer_div_0 = L.tileLayer(" in leaflet

    assert ".addTo(map_div);" in leaflet


def test_layer_control():
    import folium

    from streamlit_folium import generate_leaflet_string

    map = folium.Map()
    folium.LayerControl().add_to(map)
    map.render()
    leaflet = generate_leaflet_string(map)
    assert "var tile_layer_div_0 = L.tileLayer(" in leaflet
    assert '"openstreetmap" : tile_layer_div_0,' in leaflet


def test_draw_support():
    import folium
    from folium.plugins import Draw

    from streamlit_folium import _get_map_string

    map = folium.Map()
    Draw(export=True).add_to(map)
    map.render()
    leaflet = _get_map_string(map)
    assert "map_div.on(L.Draw.Event.CREATED, function(e) {" in leaflet
    assert "drawnItems.addLayer(layer);" in leaflet

    assert (
        """map_div.on('draw:created', function(e) {
                drawnItems.addLayer(e.layer);
            });"""
        in leaflet
    )

    assert (
        """var draw_control_div_1 = new L.Control.Draw(
                options
            ).addTo( map_div );"""
        in leaflet
    )

    assert "alert" not in leaflet


def test_map_id():
    import folium

    from streamlit_folium import _get_map_string

    map = folium.Map()
    leaflet = _get_map_string(map)
    assert "var map_div = L.map(" in leaflet


def test_feature_group():
    import folium

    from streamlit_folium import _get_feature_group_string

    fg = folium.FeatureGroup()
    m = folium.Map()

    fg_str = _get_feature_group_string(fg, m)

    assert "var feature_group_feature_group_0 = L.featureGroup(" in fg_str
    assert ".addTo(map_div);" in fg_str


def test_dual_map():
    import folium.plugins

    from streamlit_folium import _get_map_string

    dual_map = folium.plugins.DualMap()
    dual_map.render()
    map_str = _get_map_string(dual_map)

    assert "var map_div = L.map(" in map_str
    assert "var map_div2 = L.map(" in map_str


def test_vector_grid():
    import folium
    from folium.plugins import VectorGridProtobuf

    from streamlit_folium import _get_map_string

    m = folium.Map()
    url = "https://free-{s}.tilehosting.com/data/v3/{z}/{x}/{y}.pbf?token={token}"
    VectorGridProtobuf(url, "test").add_to(m)
    leaflet = _get_map_string(m)
    assert "var vector_grid_protobuf_div_1 = L.vectorGrid.protobuf(" in leaflet



from __future__ import annotations

from contextlib import contextmanager
from time import sleep

import pytest
from playwright.sync_api import Page, Response, expect

LOCAL_TEST = False

PORT = "8503" if LOCAL_TEST else "8699"


@pytest.fixture(scope="module", autouse=True)
def _before_module():
    # Run the streamlit app before each module
    with run_streamlit():
        yield


@pytest.fixture(autouse=True)
def _before_test(page: Page):
    page.goto(f"localhost:{PORT}")
    page.set_viewport_size({"width": 2000, "height": 2000})
    expect.set_options(timeout=5_000)


# Take screenshot of each page if there are failures for this session
@pytest.fixture(autouse=True)
def _after_test(page: Page, request):
    yield
    if request.node.rep_call.failed:
        page.screenshot(path=f"screenshot-{request.node.name}.png", full_page=True)


@contextmanager
def run_streamlit():
    """Run the streamlit app at examples/streamlit_app.py on port 8599"""
    import subprocess

    if LOCAL_TEST:
        try:
            yield 1
        finally:
            pass
    else:
        p = subprocess.Popen(
            [
                "streamlit",
                "run",
                "examples/streamlit_app.py",
                "--server.port",
                PORT,
                "--server.headless",
                "true",
            ]
        )

        sleep(5)

        try:
            yield 1
        finally:
            p.kill()


def click_button_or_marker(page: Page, nth: int = 0, locator: str | None = None):
    """For some reason, there's a discrepancy between how the map markers are
    selectable locally and on github actions, perhaps related some error in loading
    the actual marker images. This tries both ways to select a marker"""

    frame = page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]')
    if locator is not None:
        frame = frame.locator(locator)
    try:
        frame.get_by_role("button", name="Marker").nth(nth).click(timeout=5_000)
    except Exception:
        frame.get_by_role("img").nth(nth).click(timeout=5_000)


def test_marker_click(page: Page):
    def check_for_404(response: Response):
        if not response.ok:
            print(response)
            print(response.text())
            print(response.url)
            print(response.status)
            raise Exception("404")

    page.on("response", check_for_404)

    # Check page title
    expect(page).to_have_title("streamlit-folium documentation")

    expect(page.get_by_text('"last_object_clicked":NULL')).to_be_visible()

    # Click marker
    try:
        click_button_or_marker(page)
    except Exception as e:
        page.screenshot(path="screenshot-test-marker-click.png", full_page=True)
        raise e

    expect(page.get_by_text('"last_object_clicked":NULL')).to_be_hidden()


def test_draw(page: Page):
    # Test draw support
    page.get_by_role("link", name="draw support").click()
    # Click again to see if it resolves timeout issues
    page.get_by_role("link", name="draw support").click()

    expect(page).to_have_title("streamlit-folium documentation: Draw Support")

    page.frame_locator(
        'internal:attr=[title="streamlit_folium.st_folium"i]'
    ).get_by_role("link", name="Draw a polygon").click()

    # Should be no drawings
    expect(page.get_by_text('"all_drawings":NULL')).to_be_visible()

    page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]').get_by_role(
        "link", name="Draw a marker"
    ).click()
    page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]').locator(
        ".leaflet-marker-icon"
    ).first.click()
    page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]').locator(
        "#map_div"
    ).click()

    # Should be one item in drawings after having placed a marker
    expect(page.get_by_text('"all_drawings":NULL')).to_be_hidden()


def test_limit_data(page: Page):
    # Test limit data support
    page.get_by_role("link", name="limit data return").click()
    # Click again to see if it resolves timeout issues
    page.get_by_role("link", name="limit data return").click()

    expect(page).to_have_title("streamlit-folium documentation: Limit Data Return")

    expect(page.get_by_text('{"last_object_clicked":NULL}')).to_be_visible()

    # Click marker
    click_button_or_marker(page, 2)

    # Have to click a second time for some reason, maybe because it doesn't load right
    # away
    click_button_or_marker(page, 2)

    expect(page.get_by_text('{"last_object_clicked":{"lat":39.96')).to_be_visible()


def test_dual_map(page: Page):
    page.get_by_role("link", name="misc examples").click()
    # Click again to see if it resolves timeout issues
    page.get_by_role("link", name="misc examples").click()

    expect(page).to_have_title("streamlit-folium documentation: Misc Examples")

    page.locator("label").filter(has_text="Dual map").click()
    page.locator("label").filter(has_text="Dual map").click()

    # Click marker on left map
    try:
        click_button_or_marker(page, 0, "#map_div")
        click_button_or_marker(page, 0, "#map_div2")
    except Exception as e:
        page.screenshot(path="screenshot-dual-map.png", full_page=True)
        raise e


def test_vector_grid(page: Page):
    page.get_by_role("link", name="vector grid").click()
    page.get_by_role("link", name="vector grid").click()

    expect(page).to_have_title("streamlit-folium documentation: Vector Grid")

    page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]').locator(
        ".leaflet-marker-icon"
    ).click()


def test_tooltip_click(page: Page):
    expect(page.get_by_text('"last_object_clicked_tooltip":NULL')).to_be_visible()

    # Click marker on map
    click_button_or_marker(page)

    expect(
        page.get_by_text('"last_object_clicked_tooltip":"Liberty Bell"')
    ).to_be_visible()


def test_popup_text(page: Page):
    page.get_by_role("link", name="simple popup").click()
    page.get_by_role("link", name="simple popup").click()

    expect(page.get_by_text("Popup: None")).to_be_visible()
    expect(page.get_by_text("Tooltip: None")).to_be_visible()

    click_button_or_marker(page)

    try:
        expect(page.get_by_text("Popup: Popup!")).to_be_visible()
        expect(page.get_by_text("Tooltip: Tooltip!")).to_be_visible()
    except Exception as e:
        page.screenshot(path="screenshot-popup.png")
        raise e


def test_return_on_hover(page: Page):
    page.get_by_role("link", name="simple popup").click()
    page.get_by_role("link", name="simple popup").click()

    expect(page.get_by_text("Popup: None")).to_be_visible()
    expect(page.get_by_text("Tooltip: None")).to_be_visible()

    page.get_by_text("Return on hover?").click()

    click_button_or_marker(page, 1)

    try:
        expect(page.get_by_text("Popup: Popup 2!")).to_be_visible()
        expect(page.get_by_text("Tooltip: Tooltip 2!")).to_be_visible()
    except Exception as e:
        page.screenshot(path="screenshot-popup2.png")
        raise e


def test_responsiveness(page: Page):
    page.get_by_role("link", name="responsive").click()
    page.get_by_role("link", name="responsive").click()

    page.set_viewport_size({"width": 500, "height": 3000})

    try:
        initial_bbox = (
            page.frame_locator("iframe").nth(2).locator("#map_div").bounding_box()
        )
    except Exception as e:
        page.screenshot(path="screenshot-responsive.png", full_page=True)
        raise e

    page.set_viewport_size({"width": 1000, "height": 3000})

    sleep(1)

    new_bbox = page.query_selector_all("iframe")[2].bounding_box()

    print(initial_bbox)
    print(new_bbox)

    assert initial_bbox is not None

    assert new_bbox is not None

    assert new_bbox["width"] > initial_bbox["width"] + 300

    # Check that the iframe is reasonably tall, which makes sure it hasn't failed to
    # render at all
    assert new_bbox["height"] > 100

    page.set_viewport_size({"width": 2000, "height": 2000})


def test_geojson_styles(page: Page):
    page.get_by_role("link", name="geojson styles").click()
    page.get_by_role("link", name="geojson styles").click()

    page.get_by_text("Show generated code").click()
    expect(page.get_by_text('"fillOpacity"')).to_be_visible()


def test_grouped_layer_control(page: Page):
    page.get_by_role("link", name="grouped layer control").click()
    page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]').locator(
        "label"
    ).filter(has_text="g2").click()
    page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]').get_by_label(
        "g2"
    ).check()


def test_geojson_popup(page: Page):
    page.get_by_role("link", name="geojson popup").click()

    expect(page.get_by_text("AttributeError")).to_be_hidden()


def test_dynamic_feature_group_update(page: Page):
    page.get_by_role("link", name="dynamic updates").click()
    page.get_by_text("Show generated code").click()

    # Test showing only Parcel layer
    page.get_by_test_id("stRadio").get_by_text("Parcels").click()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"] >> nth=1')
        .locator("path")
        .first
    ).to_be_visible()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"] >> nth=1')
        .get_by_role("img")
        .locator("svg")
    ).to_be_hidden()
    expect(
        page.get_by_text('"fillColor"')
    ).to_be_visible()  # fillColor only present in parcel style
    expect(
        page.get_by_text('"dashArray"')
    ).to_be_hidden()  # dashArray only present in building style

    # Test showing only Building layer
    page.get_by_test_id("stRadio").get_by_text("Buildings").click()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"] >> nth=1')
        .locator("path")
        .first
    ).to_be_visible()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"] >> nth=1')
        .get_by_role("img")
        .locator("svg")
    ).to_be_hidden()
    expect(page.get_by_text("fillColor")).to_be_hidden()
    expect(page.get_by_text("dashArray")).to_be_visible()

    # Test showing no layers
    page.get_by_test_id("stRadio").get_by_text("None").click()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"] >> nth=1')
        .get_by_role("img")
        .locator("svg")
    ).to_be_hidden()
    expect(page.get_by_text("fillColor")).to_be_hidden()
    expect(page.get_by_text("dashArray")).to_be_hidden()

    # Test showing both layers
    page.get_by_test_id("stRadio").get_by_text("Both").click()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"] >> nth=1')
        .locator("path")
        .first
    ).to_be_visible()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"] >> nth=1')
        .locator("path")
        .nth(1)
    ).to_be_visible()
    expect(page.get_by_text("fillColor")).to_be_visible()
    expect(page.get_by_text("dashArray")).to_be_visible()


def test_layer_control_dynamic_update(page: Page):
    page.get_by_role("link", name="dynamic layer control").click()

    page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]').get_by_text(
        "Parcels"
    ).click()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]').get_by_text(
            "Parcels"
        )
    ).not_to_be_checked()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]')
        .get_by_role("img")
        .locator("svg")
    ).to_be_hidden()
    expect(page.get_by_text("dashArray")).to_be_hidden()

    page.get_by_test_id("stRadio").get_by_text("Both").click()
    page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]').get_by_text(
        "Parcels"
    ).click()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]').get_by_text(
            "Parcels"
        )
    ).not_to_be_checked()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]').get_by_text(
            "Buildings"
        )
    ).to_be_checked()
    expect(
        page.frame_locator('iframe[title="streamlit_folium\\.st_folium"]')
        .get_by_role("img")
        .locator("path")
    ).to_be_visible()


def test_frame_height_matches_content_height(page: Page):
    """Make sure that the frame height matches the document height to confirm that
    Streamlit.setFrameHeight was called correctly."""

    iframe = page.get_by_test_id("stCustomComponentV1")
    iframe_html = page.frame_locator("[data-testid=stCustomComponentV1]").locator(
        "html"
    )

    # Wait for the iframe to load and have a height - otherwise the test will be flaky
    page.wait_for_function(
        "el => window.getComputedStyle(el).height !== '0px'",
        arg=iframe.element_handle(),
    )
    sleep(0.5)

    # Now make sure that the heights match
    iframe_height = iframe.evaluate("el => window.getComputedStyle(el).height")
    iframe_html_height = iframe_html.evaluate(
        "el => window.getComputedStyle(el).height"
    )

    assert iframe_height == iframe_html_height



import setuptools

setuptools.setup(
    name="streamlit_folium",
    version="0.24.1",
    author="Randy Zwitch",
    author_email="rzwitch@gmail.com",
    description="Render Folium objects in Streamlit",
    license="MIT",
    long_description="",
    long_description_content_type="text/plain",
    url="https://github.com/randyzwitch/streamlit-folium",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[],
    python_requires=">=3.9",
    install_requires=["streamlit>=1.35.0", "folium>=0.13,!=0.15.0", "jinja2", "branca"],
)
